@@ -51,46 +51,47 @@ public final class Server {
         // not instantiable
     }
 
-    private static void initAndCleanUp(Options options) {
-        boolean mustDisableShowTouchesOnCleanUp = false;
-        int restoreStayOn = -1;
-        boolean restoreNormalPowerMode = options.getControl(); // only restore power mode if control is enabled
-        if (options.getShowTouches() || options.getStayAwake()) {
-            if (options.getShowTouches()) {
-                try {
-                    String oldValue = Settings.getAndPutValue(Settings.TABLE_SYSTEM, "show_touches", "1");
-                    // If "show touches" was disabled, it must be disabled back on clean up
-                    mustDisableShowTouchesOnCleanUp = !"1".equals(oldValue);
-                } catch (SettingsException e) {
-                    Ln.e("Could not change \"show_touches\"", e);
-                }
-            }
+    private static void initAndCleanUp(Options options, CleanUp cleanUp) {
+        // This method is called from its own thread, so it may only configure cleanup actions which are NOT dynamic (i.e. they are configured once
+        // and for all, they cannot be changed from another thread)
 
-            if (options.getStayAwake()) {
-                int stayOn = BatteryManager.BATTERY_PLUGGED_AC | BatteryManager.BATTERY_PLUGGED_USB | BatteryManager.BATTERY_PLUGGED_WIRELESS;
-                try {
-                    String oldValue = Settings.getAndPutValue(Settings.TABLE_GLOBAL, "stay_on_while_plugged_in", String.valueOf(stayOn));
-                    try {
-                        restoreStayOn = Integer.parseInt(oldValue);
-                        if (restoreStayOn == stayOn) {
-                            // No need to restore
-                            restoreStayOn = -1;
-                        }
-                    } catch (NumberFormatException e) {
-                        restoreStayOn = 0;
+        if (options.getShowTouches()) {
+            try {
+                String oldValue = Settings.getAndPutValue(Settings.TABLE_SYSTEM, "show_touches", "1");
+                // If "show touches" was disabled, it must be disabled back on clean up
+                if (!"1".equals(oldValue)) {
+                    if (!cleanUp.setDisableShowTouches(true)) {
+                        Ln.e("Could not disable show touch on exit");
                     }
-                } catch (SettingsException e) {
-                    Ln.e("Could not change \"stay_on_while_plugged_in\"", e);
                 }
+            } catch (SettingsException e) {
+                Ln.e("Could not change \"show_touches\"", e);
             }
         }
 
-        if (options.getCleanup()) {
+        if (options.getStayAwake()) {
+            int stayOn = BatteryManager.BATTERY_PLUGGED_AC | BatteryManager.BATTERY_PLUGGED_USB | BatteryManager.BATTERY_PLUGGED_WIRELESS;
             try {
-                CleanUp.configure(options.getDisplayId(), restoreStayOn, mustDisableShowTouchesOnCleanUp, restoreNormalPowerMode,
-                        options.getPowerOffScreenOnClose());
-            } catch (IOException e) {
-                Ln.e("Could not configure cleanup", e);
+                String oldValue = Settings.getAndPutValue(Settings.TABLE_GLOBAL, "stay_on_while_plugged_in", String.valueOf(stayOn));
+                try {
+                    int restoreStayOn = Integer.parseInt(oldValue);
+                    if (restoreStayOn != stayOn) {
+                        // Restore only if the current value is different
+                        if (!cleanUp.setRestoreStayOn(restoreStayOn)) {
+                            Ln.e("Could not restore stay on on exit");
+                        }
+                    }
+                } catch (NumberFormatException e) {
+                    // ignore
+                }
+            } catch (SettingsException e) {
+                Ln.e("Could not change \"stay_on_while_plugged_in\"", e);
+            }
+        }
+
+        if (options.getPowerOffScreenOnClose()) {
+            if (!cleanUp.setPowerOffScreen(true)) {
+                Ln.e("Could not power off screen on exit");
             }
         }
     }
@@ -101,7 +102,13 @@ public final class Server {
             throw new ConfigurationException("Camera mirroring is not supported");
         }
 
-        Thread initThread = startInitThread(options);
+        CleanUp cleanUp = null;
+        Thread initThread = null;
+
+        if (options.getCleanup()) {
+            cleanUp = CleanUp.configure(options.getDisplayId());
+            initThread = startInitThread(options, cleanUp);
+        }
 
         int scid = options.getScid();
         boolean tunnelForward = options.isTunnelForward();
@@ -124,8 +131,12 @@ public final class Server {
             }
 
             if (control) {
-                Controller controller = new Controller(device, connection, options.getClipboardAutosync(), options.getPowerOn());
-                device.setClipboardListener(text -> controller.getSender().pushClipboardText(text));
+                ControlChannel controlChannel = connection.getControlChannel();
+                Controller controller = new Controller(device, controlChannel, cleanUp, options.getClipboardAutosync(), options.getPowerOn());
+                device.setClipboardListener(text -> {
+                    DeviceMessage msg = DeviceMessage.createClipboard(text);
+                    controller.getSender().send(msg);
+                });
                 asyncProcessors.add(controller);
             }
 
@@ -167,7 +178,9 @@ public final class Server {
 
             completion.await();
         } finally {
-            initThread.interrupt();
+            if (initThread != null) {
+                initThread.interrupt();
+            }
             for (AsyncProcessor asyncProcessor : asyncProcessors) {
                 asyncProcessor.stop();
             }
@@ -175,7 +188,9 @@ public final class Server {
             connection.shutdown();
 
             try {
-                initThread.join();
+                if (initThread != null) {
+                    initThread.join();
+                }
                 for (AsyncProcessor asyncProcessor : asyncProcessors) {
                     asyncProcessor.join();
                 }
@@ -187,8 +202,8 @@ public final class Server {
         }
     }
 
-    private static Thread startInitThread(final Options options) {
-        Thread thread = new Thread(() -> initAndCleanUp(options), "init-cleanup");
+    private static Thread startInitThread(final Options options, final CleanUp cleanUp) {
+        Thread thread = new Thread(() -> initAndCleanUp(options, cleanUp), "init-cleanup");
         thread.start();
         return thread;
     }
